@@ -5,16 +5,23 @@ import calendar
 from decimal import Decimal
 from dateutil import relativedelta
 from dateutil import rrule
+from itertools import groupby
+from cached_property import cached_property
 
-from trytond.model import Workflow, ModelSQL, ModelView, fields
+from trytond.model import Workflow, ModelSQL, ModelView, fields, Unique
 from trytond.pyson import Eval, Bool, If
 from trytond.pool import Pool
 from trytond.transaction import Transaction
-from trytond.wizard import Wizard, StateView, StateTransition, Button
+from trytond.wizard import (Wizard, StateView, StateTransition, StateReport,
+    Button)
+from trytond.tools import grouped_slice
+from trytond.modules.company import CompanyReport
 
 __all__ = ['Asset', 'AssetLine', 'AssetUpdateMove',
     'CreateMovesStart', 'CreateMoves',
-    'UpdateAssetStart', 'UpdateAssetShowDepreciation', 'UpdateAsset']
+    'UpdateAssetStart', 'UpdateAssetShowDepreciation', 'UpdateAsset',
+    'AssetDepreciationTable',
+    'PrintDepreciationTableStart', 'PrintDepreciationTable']
 
 
 def date2datetime(date):
@@ -24,8 +31,8 @@ def date2datetime(date):
 class Asset(Workflow, ModelSQL, ModelView):
     'Asset'
     __name__ = 'account.asset'
-    _rec_name = 'reference'
-    reference = fields.Char('Reference', readonly=True, select=True)
+    _rec_name = 'number'
+    number = fields.Char('Number', readonly=True, select=True)
     product = fields.Many2One('product.product', 'Product', required=True,
         states={
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
@@ -42,12 +49,16 @@ class Asset(Workflow, ModelSQL, ModelView):
                 ('product', '=', -1),
                 ('product', '=', Eval('product', -1)),
                 ),
-            ('invoice.type', '=', 'in_invoice'),
+            ('invoice.type', '=', 'in'),
+            ['OR',
+                ('company', '=', Eval('company', -1)),
+                ('invoice.company', '=', Eval('company', -1)),
+                ],
             ],
         states={
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
-        depends=['product', 'state'])
+        depends=['product', 'state', 'company'])
     customer_invoice_line = fields.Function(fields.Many2One(
             'account.invoice.line', 'Customer Invoice Line'),
         'get_customer_invoice_line')
@@ -107,13 +118,15 @@ class Asset(Workflow, ModelSQL, ModelView):
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
         required=True,
-        depends=['state'])
+        domain=[('start_date', '<=', Eval('end_date', None))],
+        depends=['state', 'end_date'])
     end_date = fields.Date('End Date',
         states={
             'readonly': (Eval('lines', [0]) | (Eval('state') != 'draft')),
             },
         required=True,
-        depends=['state'])
+        domain=[('end_date', '>=', Eval('start_date', None))],
+        depends=['state', 'start_date'])
     depreciation_method = fields.Selection([
             ('linear', 'Linear'),
             ], 'Depreciation Method',
@@ -138,23 +151,32 @@ class Asset(Workflow, ModelSQL, ModelView):
             ], 'State', readonly=True)
     lines = fields.One2Many('account.asset.line', 'asset', 'Lines',
         readonly=True)
-    move = fields.Many2One('account.move', 'Account Move', readonly=True)
+    move = fields.Many2One('account.move', 'Account Move', readonly=True,
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
+        depends=['company'])
     update_moves = fields.Many2Many('account.asset-update-account.move',
         'asset', 'move', 'Update Moves', readonly=True,
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
         states={
             'invisible': ~Eval('update_moves'),
-            })
+            },
+        depends=['company'])
     comment = fields.Text('Comment')
 
     @classmethod
     def __setup__(cls):
         super(Asset, cls).__setup__()
+        table = cls.__table__()
         cls._sql_constraints = [
-            ('invoice_line_uniq', 'UNIQUE(supplier_invoice_line)',
-                'Supplier Invoice Line can be used only once on asset!'),
+            ('invoice_line_uniq', Unique(table, table.supplier_invoice_line),
+                'Supplier Invoice Line can be used only once on asset.'),
             ]
         cls._error_messages.update({
-                'delete_draft': 'Asset "%s" must be in draft to be deleted!',
+                'delete_draft': 'Asset "%s" must be in draft to be deleted.',
                 })
         cls._transitions |= set((
                 ('draft', 'running'),
@@ -163,23 +185,37 @@ class Asset(Workflow, ModelSQL, ModelView):
         cls._buttons.update({
                 'run': {
                     'invisible': Eval('state') != 'draft',
+                    'depends': ['state'],
                     },
                 'close': {
                     'invisible': Eval('state') != 'running',
+                    'depends': ['state'],
                     },
                 'create_lines': {
                     'invisible': (Eval('lines', [])
                         | (Eval('state') != 'draft')),
+                    'depends': ['state'],
                     },
                 'clear_lines': {
                     'invisible': (~Eval('lines', [0])
                         | (Eval('state') != 'draft')),
+                    'depends': ['state'],
                     },
                 'update': {
                     'invisible': Eval('state') != 'running',
+                    'depends': ['state'],
                     },
                 })
 
+    @classmethod
+    def __register__(cls, module_name):
+        table_h = cls.__table_handler__(module_name)
+
+        # Migration from 3.8: rename reference into number
+        if table_h.column_exist('reference'):
+            table_h.column_rename('reference', 'number')
+        super(Asset, cls).__register__(module_name)
+
     @staticmethod
     def default_state():
         return 'draft'
@@ -222,39 +258,35 @@ class Asset(Workflow, ModelSQL, ModelView):
         Currency = pool.get('currency.currency')
         Unit = Pool().get('product.uom')
 
-        new_values = {}
         if not self.supplier_invoice_line:
-            new_values['quantity'] = None
-            new_values['value'] = None
-            new_values['start_date'] = self.default_start_date()
-            return new_values
+            self.quantity = None
+            self.value = None
+            self.start_date = self.default_start_date()
+            return
 
         invoice_line = self.supplier_invoice_line
         invoice = invoice_line.invoice
         if invoice.company.currency != invoice.currency:
             with Transaction().set_context(date=invoice.currency_date):
-                new_values['value'] = Currency.compute(
+                self.value = Currency.compute(
                     invoice.currency, invoice_line.amount,
                     invoice.company.currency)
         else:
-            new_values['value'] = invoice_line.amount
+            self.value = invoice_line.amount
         if invoice.invoice_date:
-            new_values['purchase_date'] = invoice.invoice_date
-            new_values['start_date'] = invoice.invoice_date
+            self.purchase_date = invoice.invoice_date
+            self.start_date = invoice.invoice_date
             if invoice_line.product.depreciation_duration:
                 duration = relativedelta.relativedelta(
-                    months=int(invoice_line.product.depreciation_duration),
+                    months=invoice_line.product.depreciation_duration,
                     days=-1)
-                new_values['end_date'] = new_values['start_date'] + duration
+                self.end_date = self.start_date + duration
 
         if not self.unit:
-            new_values['quantity'] = invoice_line.quantity
-            return new_values
-
-        new_values['quantity'] = Unit.compute_qty(invoice_line.unit,
-            invoice_line.quantity, self.unit)
-
-        return new_values
+            self.quantity = invoice_line.quantity
+        else:
+            self.quantity = Unit.compute_qty(invoice_line.unit,
+                invoice_line.quantity, self.unit)
 
     @fields.depends('product')
     def on_change_with_unit(self):
@@ -270,7 +302,8 @@ class Asset(Workflow, ModelSQL, ModelView):
 
     @fields.depends('end_date', 'product', 'start_date')
     def on_change_with_end_date(self):
-        if all(getattr(self, k, None) for k in ('product', 'start_date')):
+        if (all(getattr(self, k, None) for k in ('product', 'start_date'))
+                and not self.end_date):
             if self.product.depreciation_duration:
                 duration = relativedelta.relativedelta(
                     months=int(self.product.depreciation_duration), days=-1)
@@ -297,6 +330,10 @@ class Asset(Workflow, ModelSQL, ModelView):
         Returns all the remaining dates at which asset depreciation movement
         will be issued.
         """
+        pool = Pool()
+        Config = pool.get('account.configuration')
+        config = Config(1)
+
         start_date = max([self.start_date] + [l.date for l in self.lines])
         delta = relativedelta.relativedelta(self.end_date, start_date)
         # dateutil >= 2.0 has replace __nonzero__ by __bool__ which doesn't
@@ -305,10 +342,11 @@ class Asset(Workflow, ModelSQL, ModelView):
             return [self.end_date]
         if self.frequency == 'monthly':
             rule = rrule.rrule(rrule.MONTHLY, dtstart=self.start_date,
-                bymonthday=-1)
+                bymonthday=int(config.asset_bymonthday))
         elif self.frequency == 'yearly':
             rule = rrule.rrule(rrule.YEARLY, dtstart=self.start_date,
-                bymonth=12, bymonthday=-1)
+                bymonth=int(config.asset_bymonth),
+                bymonthday=int(config.asset_bymonthday))
         dates = [d.date()
             for d in rule.between(date2datetime(start_date),
                 date2datetime(self.end_date))]
@@ -363,11 +401,10 @@ class Asset(Workflow, ModelSQL, ModelView):
         asset_line = None
         for date in dates:
             depreciation = self.compute_depreciation(date, dates)
-            with Transaction().set_user(0, set_context=True):
-                amounts[date] = asset_line = Line(
-                    acquired_value=self.value,
-                    depreciable_basis=amount,
-                    )
+            amounts[date] = asset_line = Line(
+                acquired_value=self.value,
+                depreciable_basis=amount,
+                )
             if depreciation > residual_value:
                 asset_line.depreciation = residual_value
                 asset_line.accumulated_depreciation = (
@@ -384,7 +421,7 @@ class Asset(Workflow, ModelSQL, ModelView):
             if residual_value > 0 and asset_line is not None:
                 asset_line.depreciation += residual_value
                 asset_line.accumulated_depreciation += residual_value
-        for asset_line in amounts.itervalues():
+        for asset_line in amounts.values():
             asset_line.actual_value = (self.value -
                 asset_line.accumulated_depreciation)
         return amounts
@@ -392,13 +429,17 @@ class Asset(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     def create_lines(cls, assets):
+        pool = Pool()
+        Line = pool.get('account.asset.line')
         cls.clear_lines(assets)
 
+        lines = []
         for asset in assets:
-            for date, line in asset.depreciate().iteritems():
+            for date, line in asset.depreciate().items():
                 line.asset = asset.id
                 line.date = date
-                line.save()
+                lines.append(line)
+        Line.save(lines)
 
     @classmethod
     @ModelView.button
@@ -427,7 +468,7 @@ class Asset(Workflow, ModelSQL, ModelView):
         MoveLine = pool.get('account.move.line')
 
         period_id = Period.find(self.company.id, line.date)
-        with Transaction().set_user(0, set_context=True):
+        with Transaction().set_context(date=line.date):
             expense_line = MoveLine(
                 credit=0,
                 debit=line.depreciation,
@@ -439,13 +480,14 @@ class Asset(Workflow, ModelSQL, ModelView):
                 account=self.product.account_depreciation_used,
                 )
 
-            return Move(
-                origin=line,
-                period=period_id,
-                journal=self.account_journal,
-                date=line.date,
-                lines=[expense_line, depreciation_line],
-                )
+        return Move(
+            company=self.company,
+            origin=line,
+            period=period_id,
+            journal=self.account_journal,
+            date=line.date,
+            lines=[expense_line, depreciation_line],
+            )
 
     @classmethod
     def create_moves(cls, assets, date):
@@ -455,26 +497,22 @@ class Asset(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Move = pool.get('account.move')
         Line = pool.get('account.asset.line')
-        cursor = Transaction().cursor
 
         moves = []
         lines = []
-        for i in range(0, len(assets), cursor.IN_MAX):
-            asset_ids = [a.id for a in assets[i:i + cursor.IN_MAX]]
+        for asset_ids in grouped_slice(assets):
             lines += Line.search([
-                    ('asset', 'in', asset_ids),
+                    ('asset', 'in', list(asset_ids)),
                     ('date', '<=', date),
                     ('move', '=', None),
                     ])
         for line in lines:
-            move = line.asset.get_move(line)
-            move.save()
-            moves.append(move)
-            Line.write([line], {
-                    'move': move.id,
-                    })
-        with Transaction().set_user(0, set_context=True):
-            Move.post(moves)
+            moves.append(line.asset.get_move(line))
+        Move.save(moves)
+        for move, line in zip(moves, lines):
+            line.move = move
+        Line.save(lines)
+        Move.post(moves)
 
     def get_closing_move(self, account):
         """
@@ -489,46 +527,44 @@ class Asset(Workflow, ModelSQL, ModelView):
         date = Date.today()
         period_id = Period.find(self.company.id, date)
         if self.supplier_invoice_line:
-            account_asset = self.supplier_invoice_line.account
+            account_asset = self.supplier_invoice_line.account.current()
         else:
             account_asset = self.product.account_asset_used
 
-        with Transaction().set_user(0, set_context=True):
-            asset_line = MoveLine(
-                debit=0,
-                credit=self.value,
-                account=account_asset,
-                )
-            depreciation_line = MoveLine(
-                debit=self.get_depreciated_amount(),
-                credit=0,
-                account=self.product.account_depreciation_used,
-                )
+        asset_line = MoveLine(
+            debit=0,
+            credit=self.value,
+            account=account_asset,
+            )
+        depreciation_line = MoveLine(
+            debit=self.get_depreciated_amount(),
+            credit=0,
+            account=self.product.account_depreciation_used,
+            )
         lines = [asset_line, depreciation_line]
         square_amount = asset_line.credit - depreciation_line.debit
         if square_amount:
             if not account:
                 account = self.product.account_revenue_used
-            with Transaction().set_user(0, set_context=True):
-                counter_part_line = MoveLine(
-                    debit=square_amount if square_amount > 0 else 0,
-                    credit=-square_amount if square_amount < 0 else 0,
-                    account=account,
-                    )
-            lines.append(counter_part_line)
-        with Transaction().set_user(0, set_context=True):
-            return Move(
-                origin=self,
-                period=period_id,
-                journal=self.account_journal,
-                date=date,
-                lines=lines,
+            counter_part_line = MoveLine(
+                debit=square_amount if square_amount > 0 else 0,
+                credit=-square_amount if square_amount < 0 else 0,
+                account=account,
                 )
+            lines.append(counter_part_line)
+        return Move(
+            company=self.company,
+            origin=self,
+            period=period_id,
+            journal=self.account_journal,
+            date=date,
+            lines=lines,
+            )
 
     @classmethod
-    def set_reference(cls, assets):
+    def set_number(cls, assets):
         '''
-        Fill the reference field with asset sequence.
+        Fill the number field with asset sequence.
         '''
         pool = Pool()
         Sequence = pool.get('ir.sequence')
@@ -536,18 +572,16 @@ class Asset(Workflow, ModelSQL, ModelView):
 
         config = Config(1)
         for asset in assets:
-            if asset.reference:
+            if asset.number:
                 continue
-            reference = Sequence.get_id(config.asset_sequence.id)
-            cls.write([asset], {
-                    'reference': reference,
-                    })
+            asset.number = Sequence.get_id(config.asset_sequence.id)
+        cls.save(assets)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('running')
     def run(cls, assets):
-        cls.set_reference(assets)
+        cls.set_number(assets)
         cls.create_lines(assets)
 
     @classmethod
@@ -563,22 +597,20 @@ class Asset(Workflow, ModelSQL, ModelView):
         cls.clear_lines(assets)
         moves = []
         for asset in assets:
-            move = asset.get_closing_move(account)
-            move.save()
-            moves.append(move)
-            cls.write([asset], {
-                    'move': move.id,
-                    })
-        with Transaction().set_user(0, set_context=True):
-            Move.post(moves)
+            moves.append(asset.get_closing_move(account))
+        Move.save(moves)
+        for move, asset in zip(moves, assets):
+            asset.move = move
+        cls.save(assets)
+        Move.post(moves)
 
     def get_rec_name(self, name):
-        return '%s - %s' % (self.reference, self.product.rec_name)
+        return '%s - %s' % (self.number, self.product.rec_name)
 
     @classmethod
     def search_rec_name(cls, name, clause):
         names = clause[2].split(' - ', 1)
-        res = [('reference', clause[1], names[0])]
+        res = [('number', clause[1], names[0])]
         if len(names) != 1 and names[1]:
             res.append(('product', clause[1], names[1]))
         return res
@@ -587,10 +619,10 @@ class Asset(Workflow, ModelSQL, ModelView):
     def copy(cls, assets, default=None):
         if default is None:
             default = {}
-        default = default.copy()
+        else:
+            default = default.copy()
         default.setdefault('lines', [])
-        default.setdefault('state', 'draft')
-        default.setdefault('reference', None)
+        default.setdefault('number', None)
         default.setdefault('supplier_invoice_line', None)
         return super(Asset, cls).copy(assets, default=default)
 
@@ -606,15 +638,16 @@ class AssetLine(ModelSQL, ModelView):
     'Asset Line'
     __name__ = 'account.asset.line'
     asset = fields.Many2One('account.asset', 'Asset', required=True,
-        ondelete='CASCADE')
-    date = fields.Date('Date')
+        ondelete='CASCADE', readonly=True)
+    date = fields.Date('Date', readonly=True)
     depreciation = fields.Numeric('Depreciation',
         digits=(16, Eval('_parent_asset', {}).get('currency_digits', 2)),
-        required=True)
-    acquired_value = fields.Numeric('Acquired Value')
-    depreciable_basis = fields.Numeric('Depreciable Basis')
-    actual_value = fields.Numeric('Actual Value')
-    accumulated_depreciation = fields.Numeric('Accumulated Depreciation')
+        required=True, readonly=True)
+    acquired_value = fields.Numeric('Acquired Value', readonly=True)
+    depreciable_basis = fields.Numeric('Depreciable Basis', readonly=True)
+    actual_value = fields.Numeric('Actual Value', readonly=True)
+    accumulated_depreciation = fields.Numeric(
+        'Accumulated Depreciation', readonly=True)
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
 
     @classmethod
@@ -673,7 +706,17 @@ class UpdateAssetShowDepreciation(ModelView):
     'Update Asset Show Depreciation'
     __name__ = 'account.asset.update.show_depreciation'
     amount = fields.Numeric('Amount', readonly=True)
-    date = fields.Date('Date', required=True)
+    date = fields.Date('Date', required=True,
+        domain=[
+            ('date', '>', Eval('latest_move_date')),
+            ('date', '<', Eval('next_depreciation_date')),
+            ],
+        depends=['latest_move_date', 'next_depreciation_date'],
+        help=('The date must be between the last update/depreciation date '
+            'and the next depreciation date.'))
+    latest_move_date = fields.Date('Latest Move Date', readonly=True)
+    next_depreciation_date = fields.Date('Next Depreciation Date',
+        readonly=True)
     depreciation_account = fields.Many2One('account.account',
         'Depreciation Account', readonly=True)
     counterpart_account = fields.Many2One('account.account',
@@ -713,6 +756,21 @@ class UpdateAsset(Wizard):
             return 'show_move'
         return 'create_lines'
 
+    def get_latest_move_date(self, asset):
+        previous_dates = [datetime.date.min]
+        previous_dates += [m.date for m in asset.update_moves
+            if m.state == 'posted']
+        previous_dates += [l.date for l in asset.lines
+                if l.move and l.move.state == 'posted']
+        return max(previous_dates)
+
+    def get_next_depreciation_date(self, asset):
+        next_dates = [datetime.date.max]
+        next_dates += [l.date for l in asset.lines
+            if not l.move or l.move.state != 'posted']
+
+        return min(next_dates)
+
     def default_show_move(self, fields):
         Asset = Pool().get('account.asset')
         asset = Asset(Transaction().context['active_id'])
@@ -721,6 +779,8 @@ class UpdateAsset(Wizard):
             'date': datetime.date.today(),
             'depreciation_account': asset.product.account_depreciation_used.id,
             'counterpart_account': asset.product.account_expense_used.id,
+            'latest_move_date': self.get_latest_move_date(asset),
+            'next_depreciation_date': self.get_next_depreciation_date(asset),
             }
 
     def get_move(self, asset):
@@ -729,6 +789,7 @@ class UpdateAsset(Wizard):
         Move = pool.get('account.move')
         period_id = Period.find(asset.company.id, self.show_move.date)
         return Move(
+            company=asset.company,
             origin=asset,
             journal=asset.account_journal.id,
             period=period_id,
@@ -755,6 +816,10 @@ class UpdateAsset(Wizard):
         Asset = pool.get('account.asset')
 
         asset = Asset(Transaction().context['active_id'])
+        latest_move_date = self.show_move.latest_move_date
+        next_date = self.show_move.next_depreciation_date
+        if not (latest_move_date < self.show_move.date < next_date):
+            raise ValueError('The update move date is invalid')
         move = self.get_move(asset)
         move.lines = self.get_move_lines(asset)
         move.save()
@@ -775,3 +840,202 @@ class UpdateAsset(Wizard):
                 })
         Asset.create_lines([asset])
         return 'end'
+
+
+class AssetDepreciationTable(CompanyReport):
+    'Asset Depreciation Table'
+    __name__ = 'account.asset.depreciation_table'
+
+    @classmethod
+    def get_context(cls, records, data):
+        context = super(AssetDepreciationTable, cls).get_context(records, data)
+
+        AssetDepreciation = cls.get_asset_depreciation()
+        AssetDepreciation.start_date = data['start_date']
+        AssetDepreciation.end_date = data['end_date']
+        Grouper = cls.get_grouper()
+        grouped_assets = groupby(sorted(records, key=cls.group_assets),
+            cls.group_assets)
+        context['grouped_depreciations'] = grouped_depreciations = []
+        for g_key, assets in grouped_assets:
+            depreciations = [AssetDepreciation(a) for a in assets]
+            grouped_depreciations.append(Grouper(g_key, depreciations))
+
+        return context
+
+    @staticmethod
+    def group_assets(asset):
+        return asset.product
+
+    @classmethod
+    def get_grouper(cls):
+
+        class Grouper(object):
+            def __init__(self, key, depreciations):
+                self.product = key
+                self.depreciations = depreciations
+
+        def adder(attr_name):
+            def _sum(self):
+                return sum(getattr(d, attr_name)
+                    for d in self.depreciations if getattr(d, attr_name))
+            return _sum
+
+        grouped_attributes = {
+            'start_fixed_value',
+            'value_increase',
+            'value_decrease',
+            'end_fixed_value',
+            'start_value',
+            'amortization_increase',
+            'amortization_decrease',
+            'end_value',
+            'actual_value',
+            'closing_value',
+            }
+        for attr_name in grouped_attributes:
+            setattr(Grouper, attr_name, cached_property(adder(attr_name)))
+
+        return Grouper
+
+    @classmethod
+    def get_asset_depreciation(cls):
+
+        class AssetDepreciation(object):
+            def __init__(self, asset):
+                self.asset = asset
+
+            @cached_property
+            def asset_lines(self):
+                return [l for l in self.asset.lines
+                    if self.start_date < l.date <= self.end_date]
+
+            @cached_property
+            def update_lines(self):
+                filter_ = lambda l: (l.account.kind == 'other'
+                    and self.start_date < l.move.date <= self.end_date)
+                return list(filter(filter_,
+                    (l for m in self.asset.update_moves for l in m.lines)))
+
+            @cached_property
+            def start_fixed_value(self):
+                if self.start_date < self.asset.start_date:
+                    return 0
+                value = self.asset_lines[0].acquired_value
+                date = self.asset_lines[0].date
+                for line in self.update_lines:
+                    if line.move.date < date:
+                        value += line.debit - line.credit
+                return value
+
+            @cached_property
+            def value_increase(self):
+                value = sum(l.debit - l.credit for l in self.update_lines
+                    if l.debit > l.credit)
+                if (self.asset_lines
+                        and self.start_date < self.asset.start_date):
+                    value += self.asset_lines[0].acquired_value
+                return value
+
+            @cached_property
+            def value_decrease(self):
+                return sum(l.credit - l.debit for l in self.update_lines
+                    if l.credit > l.debit)
+
+            @cached_property
+            def end_fixed_value(self):
+                if not self.asset_lines:
+                    return 0
+                value = self.asset_lines[-1].acquired_value
+                date = self.asset_lines[-1].date
+                for line in self.update_lines:
+                    if line.move.date > date:
+                        value += line.debit - line.credit
+                return value
+
+            @cached_property
+            def start_value(self):
+                if not self.asset_lines:
+                    return self.asset.value
+                return (self.asset_lines[0].actual_value
+                    + self.asset_lines[0].depreciation)
+
+            @cached_property
+            def amortization_increase(self):
+                return sum(l.depreciation for l in self.asset_lines
+                    if l.depreciation > 0)
+
+            @cached_property
+            def amortization_decrease(self):
+                return sum(l.depreciation for l in self.asset_lines
+                    if l.depreciation < 0)
+
+            @cached_property
+            def end_value(self):
+                if not self.asset_lines:
+                    return self.asset.value
+                return self.asset_lines[-1].actual_value
+
+            @cached_property
+            def actual_value(self):
+                value = self.end_value
+                if self.asset_lines:
+                    date = self.asset_lines[-1].date
+                    value += sum(l.debit - l.credit for l in self.update_lines
+                        if l.move.date > date)
+                return value
+
+            @cached_property
+            def closing_value(self):
+                if not self.asset.move:
+                    return None
+                revenue_lines = [l for l in self.asset.move.lines
+                    if l.account == self.asset.product.account_revenue_used]
+                return sum(l.debit - l.credit for l in revenue_lines)
+
+        return AssetDepreciation
+
+
+class PrintDepreciationTableStart(ModelView):
+    'Asset Depreciation Table Start'
+    __name__ = 'account.asset.print_depreciation_table.start'
+
+    start_date = fields.Date('Start Date', required=True,
+        domain=[('start_date', '<', Eval('end_date'))],
+        depends=['end_date'])
+    end_date = fields.Date('End Date', required=True,
+        domain=[('end_date', '>', Eval('start_date'))],
+        depends=['start_date'])
+
+    @staticmethod
+    def default_start_date():
+        return datetime.date.today() - relativedelta.relativedelta(years=1)
+
+    @staticmethod
+    def default_end_date():
+        return datetime.date.today()
+
+
+class PrintDepreciationTable(Wizard):
+    'Asset Depreciation Table'
+    __name__ = 'account.asset.print_depreciation_table'
+    start = StateView('account.asset.print_depreciation_table.start',
+        'account_asset.print_depreciation_table_start_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('Print', 'print_', 'tryton-print', default=True),
+            ])
+    print_ = StateReport('account.asset.depreciation_table')
+
+    def do_print_(self, action):
+        pool = Pool()
+        Asset = pool.get('account.asset')
+        assets = Asset.search([
+                ('start_date', '<', self.start.end_date),
+                ('end_date', '>', self.start.start_date),
+                ('state', '=', 'running'),
+                ])
+        return action, {
+            'ids': [a.id for a in assets],
+            'start_date': self.start.start_date,
+            'end_date': self.start.end_date,
+            }
